#include <string.h>

#include <algorithm>
#include <cassert>
#include <cstdlib>
#include <chrono>
#include <iostream>
#include <ostream>
#include <string>
#include <vector>
using namespace std;

#define NOF_ELEM 100000
#define STR_LEN  10

typedef char fixed_string[STR_LEN+1];

const char * gen_str() {
    static fixed_string s;
    for (size_t i = 0; i < sizeof(fixed_string)-1; ++i) {
        s[i] = rand() % 25 + 65;
    }
    s[sizeof(s)-1] = '\0';
    return s;
}

int cmp_str(const void * x, const void * y) { return strcmp((const char *)x, (const char *)y); }
void do_qsort(char arr[NOF_ELEM][sizeof(fixed_string)]) { qsort(arr, NOF_ELEM, sizeof(fixed_string), cmp_str); }
void do_ssort(vector<string>& arr) { sort(begin(arr), end(arr)); }
void do_ssort(vector<char *>& arr) { sort(begin(arr), end(arr), [](char * x, char *y) { return std::strcmp(x, y) < 0; }); }

int main()
{
    fixed_string * arr1 = (fixed_string *)malloc(sizeof(fixed_string) * NOF_ELEM);
    vector<string> arr2(NOF_ELEM);
    vector<char *> arr3(NOF_ELEM);

    for (size_t i = 0; i < NOF_ELEM; ++i) {
        const char * s = gen_str();
        strlcpy(arr1[i], s, sizeof(fixed_string));
        arr2[i] = s;
        arr3[i] = strdup(s);
    }
    
    auto get_time = []() { return chrono::high_resolution_clock::now(); };

    auto begin = get_time();
    do_qsort(arr1);
    cout << chrono::duration_cast<chrono::microseconds>(get_time() - begin).count() << endl;

    begin = get_time();
    do_ssort(arr2);
    cout << chrono::duration_cast<chrono::microseconds>(get_time() - begin).count() << endl;

    begin = get_time();
    do_ssort(arr3);
    cout << chrono::duration_cast<chrono::microseconds>(get_time() - begin).count() << endl;

    for (auto i = 0; i < NOF_ELEM; ++i) {
        assert(arr2[i] == arr1[i]);
        assert(!strcmp(arr1[i], arr3[i]));
    }
}
